<?php
/**
 * RokTwittie Module
 *
 * @package		Joomla
 * @subpackage	RokTwittie Module
 * @copyright Copyright (C) 2009 RocketTheme. All rights reserved.
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see RT-LICENSE.php
 * @author RocketTheme, LLC
 *
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');

class rokTwittie {
	var $url = array(
		"rate_limit" => "http://twitter.com/account/rate_limit_status.xml",
		"status" => "http://twitter.com/users/show.xml?screen_name=",
		"friends" => "http://twitter.com/statuses/friends/",
		"messages" => "http://twitter.com/statuses/user_timeline/"
	);
	var $output = array();	
	var $usernames = array();
	var $DB = array("status" => array(), "friends" => array());
	var $DBFile = "roktwittie";
	var $DBFileTypes = array("status", "friends");
	var $limits = 100;
	var $id = 0;
	
	function rokTwittie(&$params, $id, $db_folder) {
		$this->php_version = phpversion();
		
		$this->params = $params;
		$this->id = $id;
		
		$this->DBFolder = $db_folder;
		$this->DBFile = $db_folder . DS . $this->DBFile;
		$this->load_db();
		
		$this->usernames = str_replace(" ", "", $params->get("usernames", "rockettheme"));
		$this->usernames = explode(",", $this->usernames);
		
		/*if (strlen($login) && strlen($passw)) {
			foreach ($this->url as $key => $url) {
				$this->url[$key] = str_replace("http://", "http://" . $login . ":" . $passw . "@", $url);
			}
		}*/
	}
	
	function load_db(){
		for ($i = 0; $i < count($this->DBFileTypes); $i++) {
			$dbfile = $this->DBFile . "-" . $this->DBFileTypes[$i] . "-" . $this->id . ".db";

			if (file_exists($dbfile)) $this->DB[$this->DBFileTypes[$i]] = unserialize(JFile::read($dbfile));
			else return $this->write_db();
		}
	}

	function write_db($return_array = array(), $type = ""){

		if (!JFile::exists($this->DBFolder)){
			// attempt to make the dir
			JFolder::create($this->DBFolder, 0777);
		}
		
		if (!strlen($type)) {
			for ($i = 0; $i < count($this->DBFileTypes); $i++) {
				$dbfile = $this->DBFile . "-" . $this->DBFileTypes[$i] . "-" . $this->id . ".db";
				
				if (!JFile::write($dbfile, serialize(array()))){
					echo "<br />Could not save data to cache. Please make sure your cache directory exists and is writable.<br />";
				}				
			}
			
			return false;
		}
		
		$dbfile = $this->DBFile . "-" . $type . "-" . $this->id . ".db";
		if (!JFile::write($dbfile, serialize($return_array))){
			echo "<br />Could not save data to cache. Please make sure your cache directory exists and is writable.<br />";
		}
		
		return $return_array;
	}
	
	function checkErrors($string, $httpinfo) {
		if (!$httpinfo) return "Twitter was unavailable. Please try again later.";
		if (substr($string, 2, 3) != "xml") return "Data appears to be invalid.";
		else return false;
	}
	
	function getLimits() {
		$httpinfo = $this->curl($this->url["rate_limit"], $this->raw_data);
		
		if (strlen($error = $this->checkErrors($this->raw_data, $httpinfo))) return $error;
		
		if ($this->php_version < "5.1.3") {
			$this->raw_data = str_replace("-", "_", $this->raw_data);
			$xml = new JSimpleXML;
			$xml->loadString($this->raw_data);
			$xml = $xml->document;
		} else {
			$xml = simplexml_load_string($this->raw_data);
		}
		
		$this->limits =	$this->parseLimits($xml);
		
	}
	
	function parseLimits($xml) {
		if ($this->php_version < "5.1.3") {
			return array(
				"reset-time" => (string)$xml->reset_time[0]->data(),
				"reset-time-in-seconds" => (string)$xml->reset_time_in_seconds[0]->data(),
				"remaining-hits" => (string)$xml->remaining_hits[0]->data(),
				"hourly-limit" => (string)$xml->hourly_limit[0]->data()
			);
		} else {
			return array(
				"reset-time" => (string)$xml->{'reset-time'},
				"reset-time-in-seconds" => (string)$xml->{'reset-time-in-seconds'},
				"remaining-hits" => (string)$xml->{'remaining-hits'},
				"hourly-limit" => (string)$xml->{'hourly-limit'}
			);
		}
	}
	
	function parseStatus($xml) {
		if ($this->php_version < "5.1.3") {
			return array(
				"id" => (string)$xml->id[0]->data(),
				"name" => (string)$xml->name[0]->data(),
				"location" => (string)$xml->location[0]->data(),
				"description" => (string)$xml->description[0]->data(),
				"profile_image_url" => (string)$xml->profile_image_url[0]->data(),
				"url" => (string)$xml->url[0]->data(),
				"followers_count" => (string)$xml->followers_count[0]->data(),
				"friends_count" => (string)$xml->friends_count[0]->data(),
				"favourites_count" => (string)$xml->favourites_count[0]->data(),
				"statuses_count" => (string)$xml->statuses_count[0]->data()
			);
		} else {
			return array(
				"id" => (string)$xml->id,
				"name" => (string)$xml->name,
				"location" => (string)$xml->location,
				"description" => (string)$xml->description,
				"profile_image_url" => (string)$xml->profile_image_url,
				"url" => (string)$xml->url,
				"followers_count" => (string)$xml->followers_count,
				"friends_count" => (string)$xml->friends_count,
				"favourites_count" => (string)$xml->favourites_count,
				"statuses_count" => (string)$xml->statuses_count
			);
		}
	}
	
	function parseFriends($xml) {
		$friends = array();
		
		if ($this->php_version < "5.1.3") {
			$children = $xml->children();
		
			foreach ($children as $child) {
				$name = $child->screen_name[0]->data();
				$friends[$name] = array(
					"id" => (string)$child->id[0]->data(),
					"name" => (string)$child->name[0]->data(),
					"profile_image_url" => (string)$child->profile_image_url[0]->data()
				);
			}
		} else {
			$children = $xml->children();
		
			foreach ($children as $child) {
				$name = $child->screen_name;
				$friends[(string)$name] = array(
					"id" => (string)$child->id,
					"name" => (string)$child->name,
					"profile_image_url" => (string)$child->profile_image_url
				);
			}
		}
		
		return $friends;
	}
	
	function loadFromCache($type) {
		$output = array();
		foreach ($this->usernames as $user) {
			if (isset($this->DB[$type][$user])) {
				$output[$user] = $this->DB[$type][$user];
				$output[$user]["rate_limit"] = $this->limits;
			} else {
				$res = $this->makeRequest($type, true);
				$this->DB[$type][$user] = $res[$user];
				$output[$user] = $res[$user];
				$output[$user]["rate_limit"] = $this->limits;
			}
		}
		
		return $output;
	}
	
	function cacheTime($type) {
		$dbfile = $this->DBFile . "-" . $type . "-" . $this->id . ".db";
		$fileTime = filectime($dbfile);
		$now = time();
		$diff = ($now - $fileTime);
		
		if ($diff <= (((int)$this->params->get("enable_cache_time", 5)) * 60)){
			return true;
		} else {
			return false;
		}
	}
	
	function makeRequest($type, $force = false) {
		$ready_for_cache = $this->params->get("enable_cache", "1") == "1" && ($this->cacheTime($type) && count($this->DB[$type]) > 0);
		
		if (!$force && $ready_for_cache) {
			$output = $this->loadFromCache($type);
			
			return $output;			
		}
		
		$this->getLimits();

		if ($this->limits["remaining-hits"] <= "0" && $ready_for_cache) {
			$output = $this->loadFromCache($type);
			
			return $output;
		}
		
		//var_dump("$type => getting new data");
		
		
		$count = 0;
		foreach ($this->usernames as $user) {
			$httpinfo = $this->curl($this->url[$type].$user.(($type != 'status') ? '.xml' : ''), $this->raw_data);
			
			if (strlen($error = $this->checkErrors($this->raw_data, $httpinfo))) return $error;
			
			if ($this->php_version < "5.1.3") {
				$xml[$count] = new JSimpleXML;
				$xml[$count]->loadString($this->raw_data);
				
				if (isset($xml[$count]->document->error[0])) { return $xml[$count]->document->error[0]->data(); }
				
				$xml[$count] = $xml[$count]->document;
				
				switch ($type) {
					case "status": $output[$user] = $this->parseStatus($xml[$count]); break;
					case "friends": $output[$user] = $this->parseFriends($xml[$count]); break;
					case "messages": $output[$user] = $this->parseMessages($xml[$count]); break;
				}
				
			} else {
				$xml = simplexml_load_string($this->raw_data);
				if (isset($xml->error)) { return (string)$xml->error; }
				
				switch ($type) {
					case "status": $output[$user] = $this->parseStatus($xml); break;
					case "friends": $output[$user] = $this->parseFriends($xml); break;
					case "messages": $output[$user] = $this->parseMessages($xml); break;
				}
			}
			
			$count++;
		}
		
		$this->write_db((array) $output, $type);
		
		return $output;
	}
	
	function curl($url, &$output)
	{
		$login = $this->params->get("twitter_login", "");
		$passw = $this->params->get("twitter_passw", "");
		$timeout_response = $this->params->get("timeout_response", 6);
		$timeout_connect = $this->params->get("timeout_connect", 3);
		
		$output = "";
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
		curl_setopt($ch, CURLOPT_TIMEOUT, $timeout_response);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout_connect);
      	curl_setopt($ch, CURLOPT_HTTPHEADER, array('Expect:'));
		if (strlen($login) && strlen($passw)) curl_setopt($ch, CURLOPT_USERPWD, $login . ":" . $passw);
	   
		$output = curl_exec($ch);
		$httpinfo = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		
		curl_close($ch);
		
		return $httpinfo;
	}
}

?>