<?php
defined('JPATH_BASE') or die();
require_once JPATH_SITE.DS.'plugins'.DS.'system'.DS.'rokmoduleorder'.DS.'proxy.php';

class RTDocument extends Proxy
{
	protected $overrides;
	
	public function addOverride($type, $path, $class) {
		$override = array(
			'path'	=> $path,
			'class'	=> $class
		);
		
		$this->overrides[strtolower($type)] = $override;
	}
	
	public function &loadRenderer($type) {
		if (!empty($this->overrides[strtolower($type)])) {
			$override = $this->overrides[strtolower($type)];
			
			require_once($override['path']);
			$renderer = new $override['class']($this);
			return $renderer;
		} else {
			return $this->delegate->loadRenderer($type);
		}
	}

	function render($caching = false, $params = array())
	{
		// check
		$directory	= isset($params['directory']) ? $params['directory'] : 'templates';
		$template	= JFilterInput::clean($params['template'], 'cmd');
		$file		= JFilterInput::clean($params['file'], 'cmd');

		if (!file_exists( $directory.DS.$template.DS.$file) ) {
			$template = 'system';
		}

		// Parse the template INI file if it exists for parameters and insert
		// them into the template.
		if (is_readable( $directory.DS.$template.DS.'params.ini' ) )
		{
			$content = file_get_contents($directory.DS.$template.DS.'params.ini');
			$params = new JParameter($content);
		}

		// Load the language file for the template
		$lang =& JFactory::getLanguage();
		$lang->load( 'tpl_'.$template );

		// Assign the variables
		$this->delegate->template = $template;
		$this->delegate->baseurl  = JURI::base(true);
		$this->delegate->params   = $params;

		// load
		$data = $this->delegate->_loadTemplate($directory.DS.$template, $file);

		// parse
		$data = $this->_parseTemplate($data);

		JResponse::setHeader( 'Expires', gmdate( 'D, d M Y H:i:s', time() + 900 ) . ' GMT' );
		if ($mdate = $this->getModifiedDate()) {
			JResponse::setHeader( 'Last-Modified', $mdate /* gmdate( 'D, d M Y H:i:s', time() + 900 ) . ' GMT' */ );
		}
		JResponse::setHeader( 'Content-Type', $this->delegate->_mime .  '; charset=' . $this->delegate->_charset);

		return $data;
	}

	function getBuffer($type = null, $name = null, $attribs = array()) {
		$result = null;

		if ($type === null) {
			return $this->delegate->_buffer;
		}

		if(isset($this->delegate->_buffer[$type][$name])) {
			$result = $this->delegate->_buffer[$type][$name];
		}

		if ($result === false) {
			return null;
		}

		if( $renderer =& $this->loadRenderer( $type )) {
			$result = $renderer->render($name, $attribs, $result);
		}

		return $result;
	}

	function _parseTemplate($data)
	{
		$replace = array();
		$matches = array();
		if(preg_match_all('#<jdoc:include\ type="([^"]+)" (.*)\/>#iU', $data, $matches))
		{
			$matches[0] = array_reverse($matches[0]);
			$matches[1] = array_reverse($matches[1]);
			$matches[2] = array_reverse($matches[2]);

			$count = count($matches[1]);

			for($i = 0; $i < $count; $i++)
			{
				$attribs = JUtility::parseAttributes( $matches[2][$i] );
				$type  = $matches[1][$i];

				$name  = isset($attribs['name']) ? $attribs['name'] : null;
				$replace[$i] = $this->getBuffer($type, $name, $attribs);
			}

			$data = str_replace($matches[0], $replace, $data);
		}

		return $data;
	}
}
